import React, {Component} from 'react';
import R from 'ramda';
import classNames from 'classnames';
import PropTypes from 'prop-types';
import {connect} from 'react-redux';
import RU from '../../utils/ramda-utils';
import {ERROR, WARNING} from '../../utils/ci-utils';
import {ComponentNavigator} from '../playground/component-navigator/component-navigator';
import {XlReactDipReduxForm} from '../../xl-react-form/xl-react-dip-redux-form/xl-react-dip-redux-form';
import FormKitchenTestComponent from '../playground/form-kitchen-test-component';
import {getHandlers, lookupSuggestedElements} from '../handlers/predefined-handlers';
import {getCollectionOfCiHandlers} from '../dip-page/dip-utils';
import formActions from '../../xl-react-form/xl-react-dip-redux-form/xl-react-dip-redux-form-actions';
import {formInitialState} from './form-initial-state';

import './form-page.less';

const mapStateToProps = (state) => state.forms;

const mapDispatchToProps = (dispatch) => ({
    initialize: ({formId, ...rest}) => dispatch(formActions.initialize({formId, ...rest})),
    mergeContext: (formId, context) => dispatch(formActions.mergeContext(formId, context))
});

@connect(mapStateToProps, mapDispatchToProps)
export class Form extends Component {
    static propTypes = {
        initialize: PropTypes.func,
        match: PropTypes.object,
        mergeContext: PropTypes.func
    };

    componentDidMount() {
        R.forEach(this.initializeForm, R.keys(formInitialState));
    }

    initializeForm = (formId) => {
        if (formId) {
            const {metadata, model, objLabel} = formInitialState[formId];
            this.props.initialize({formId, metadata, model, objLabel});
        }
    };

    render() {
        const {match, mergeContext} = this.props;

        const formHandlers = {
            CI: getHandlers(),
            LIST_OF_CI: getCollectionOfCiHandlers,
            LIST_OF_STRING: {nameComponentList: getHandlers()},
            LOOKUP_CI: getHandlers(lookupSuggestedElements),
            SET_OF_CI: getCollectionOfCiHandlers
        };

        const warningValidator = (fieldName) => R.pipe(
            R.propOr([], 'validation-messages'),
            R.find(R.where({
                property: R.equals(fieldName),
                type: R.either(R.equals(WARNING), R.equals(ERROR))
            })),
            R.cond([
                [R.isNil, R.always({})],
                [R.T, R.pipe(
                    R.pick(['text', 'type']),
                    R.assoc('show', false),
                    R.objOf('message'),
                    R.assoc('condition', true))]
            ]));

        const vaultValidator = (fieldName, model) => {
            const isApplicable = () => R.equals(fieldName, 'vault') && RU.isDefined(model.provider);

            if (isApplicable()) {
                if (!R.pipe(
                    R.pluck('id'),
                    R.contains(model.provider)
                )(lookupSuggestedElements)) {
                    return {
                        condition: true,
                        message: {show: false, text: 'Invalid vault provider value', type: WARNING}
                    };
                }
            }

            return {condition: false};
        };

        const formValidator = (formState) => (fieldName) =>
            warningValidator(fieldName)(formState);

        const xlReactDipForm = (formId) => <XlReactDipReduxForm
            autoCleanup={false}
            dateTimeFormat={'YYYY-MM-DDTHH:mm:ss.SSS'}
            formId={formId}
            handlers={formHandlers} objLabel={'id'}
        />;

        const xlReactDipFormWithWarnings = (formId) => <XlReactDipReduxForm
            autoCleanup={false}
            formId={formId}
            handlers={formHandlers} objLabel={'id'}
            validators={[formValidator(formInitialState.dynamicFormEmptyWithWarnings)]}
        />;

        const xlReactDipFormWithLookup = (formId) => <XlReactDipReduxForm
            autoCleanup={false}
            formId={formId}
            handlers={formHandlers} objLabel={'id'}
            validators={[formValidator(formInitialState.dynamicWithLookup), vaultValidator]}
        />;

        const forms = {
            dynamicFormAsContainment: <FormKitchenTestComponent
                className="xl-react-dip-dynamic-form-as-containment"
                currentState={this.props}
                disabledToggles={['auto-focus']}
                key={'dynamicFormAsContainment'}
                label={'Form with an asContainment property'}
                modelName={'dynamicFormAsContainment'}
                updateState={mergeContext}>
                {xlReactDipForm('dynamicFormAsContainment')}
            </FormKitchenTestComponent>,
            dynamicFormDefault: <FormKitchenTestComponent
                className="xl-react-dip-dynamic-form-default"
                currentState={this.props}
                disabledToggles={['auto-focus']}
                key={'dynamicFormDefault'}
                label={'Form with default values'}
                modelName={'dynamicFormDefault'}
                updateState={mergeContext}>
                {xlReactDipForm('dynamicFormDefault')}
            </FormKitchenTestComponent>,
            dynamicFormEmpty: <FormKitchenTestComponent
                className="xl-react-dip-dynamic-form-empty"
                currentState={this.props}
                disabledToggles={['auto-focus']}
                key={'dynamicFormEmpty'}
                label={'Empty form'}
                modelName={'dynamicFormEmpty'}
                updateState={mergeContext}>
                {xlReactDipForm('dynamicFormEmpty')}
            </FormKitchenTestComponent>,
            dynamicFormEmptyWithWarnings: <FormKitchenTestComponent
                className="xl-dip-dynamic-form-empty"
                currentState={this.props}
                disabledToggles={['auto-focus']}
                key={'dynamicFormEmptyWithWarnings'}
                label={'Empty form with warnings'}
                modelName={'dynamicFormEmptyWithWarnings'}
                updateState={mergeContext}>
                {xlReactDipFormWithWarnings('dynamicFormEmptyWithWarnings')}
            </FormKitchenTestComponent>,
            dynamicFormWithCategoryOptions: <FormKitchenTestComponent
                className="xl-react-dip-dynamic-form-with-category-options"
                currentState={this.props}
                disabledToggles={['auto-focus']}
                key={'dynamicFormWithCategoryOptions'}
                label={'Form with category options'}
                modelName={'dynamicFormWithCategoryOptions'}
                updateState={mergeContext}>
                {xlReactDipForm('dynamicFormWithCategoryOptions')}
            </FormKitchenTestComponent>,
            dynamicFormWithMultipleCategories: <FormKitchenTestComponent
                className="xl-react-dip-dynamic-form-with-multiple-categories"
                currentState={this.props}
                disabledToggles={['auto-focus']}
                key={'dynamicFormWithMultipleCategories'}
                label={'Form with multiple categories'}
                modelName={'dynamicFormWithMultipleCategories'}
                updateState={mergeContext}>
                {xlReactDipForm('dynamicFormWithMultipleCategories')}
            </FormKitchenTestComponent>,
            dynamicReadonlyForm: <FormKitchenTestComponent
                className="xl-react-dip-form-readonly"
                currentState={this.props}
                disabledToggles={['auto-focus']}
                key={'dynamicReadonlyForm'}
                label={'Readonly Form'}
                modelName={'dynamicReadonlyForm'}
                updateState={mergeContext}>
                {xlReactDipForm('dynamicReadonlyForm')}
            </FormKitchenTestComponent>,
            dynamicWithLookup: <FormKitchenTestComponent
                className="xl-react-dip-dynamic-form-with-lookup-values"
                currentState={this.props}
                disabledToggles={['auto-focus']}
                key={'dynamicWithLookup'}
                label={'Form with lookup values'}
                modelName={'dynamicWithLookup'}
                updateState={mergeContext}>
                {xlReactDipFormWithLookup('dynamicWithLookup')}
            </FormKitchenTestComponent>
        };

        const isSingleItem = match && forms[match.params.name];

        const formStates = R.reduce((acc, key) => ({...acc, ...{[key]: this.props[key]}}), {}, R.keys(forms));

        return (
            <div className={classNames('form-page', {'single-item': isSingleItem})}>
                <ComponentNavigator componentType="form" state={formStates}/>
                <div className="page-content">
                    {isSingleItem ? forms[match.params.name] : R.values(forms)}
                </div>
            </div>
        );
    }
}
