import React from 'react';
import R from 'ramda';
import classNames from 'classnames';
import {localeMessages} from '../../services/locale-service';
import RU from '../../utils/ramda-utils';
import {XlReactWidgetDate} from '../../xl-react-widget/xl-react-widget-date/xl-react-widget-date';
import ToggleComponent from '../playground/toggle-component';
import DisplayEnumComponent from './display-enum-component';
import DisplaySelectComponent from './display-select-component';
import ApiButton from './api-button';

export const createExtraBindingSections = (bindingItems, model, onModelChange, updateProps, api) => {
    const createBindingSection = (sectionConfig, index) => {
        const updateComponentProps = (value) =>
            updateProps({...R.assocPath(sectionConfig.name.split('.'), value, {})});

        const onIntegerInputChange = (event) => {
            onModelChange(parseInt(event.target.value, 10));
        };

        const toggleBooleanValue = (event) => {
            sectionConfig.value = event.target.checked;
            updateComponentProps(sectionConfig.value);
        };

        const onInputChange = (event) => {
            sectionConfig.value = event.target.value;
            updateComponentProps(event.target.value);
        };

        const handleModelChange = (value) => {
            if (sectionConfig.onChangeCallApi) {
                sectionConfig.onChangeCallApi(api, value);
            } else {
                updateComponentProps(value);
            }
        };

        const createApiButton = () =>
            <ApiButton
                clazzName={'apply-value-btn'}
                handleClick={handleModelChange.bind(this, sectionConfig.value)}
                label={localeMessages.applyDefaultValueButton}
            />;

        return R.cond([
            [R.whereEq({type: 'integer'}),
                R.always(
                    <div key={index}>
                        <span className="label">{sectionConfig.label}</span>
                        <input onChange={onIntegerInputChange} type="number" value={sectionConfig.value || model}/>
                    </div>
                )],
            [R.whereEq({type: 'boolean'}),
                (config) => <ToggleComponent
                    checked={config.value}
                    clazzName={R.toLower(config.label)}
                    handleToggle={toggleBooleanValue}
                    key={index}
                    label={config.label}
                />
            ],
            [R.whereEq({type: 'date'}),
                () =>
                    <div className={sectionConfig.className} key={index}>
                        <span className="label">{sectionConfig.label}</span>
                        <XlReactWidgetDate model={sectionConfig.value} onModelChange={handleModelChange}/>
                    </div>
            ],
            [R.whereEq({type: 'enum'}),
                (config) => <DisplayEnumComponent
                    clazzName={config.clazzName}
                    currentValue={config.value}
                    key={index}
                    label={config.label}
                    listOfValues={config.enum}
                    name={config.name}
                    onInputChange={onInputChange}
                />],
            [R.whereEq({type: 'select'}),
                (config) => <DisplaySelectComponent
                    clazzName={config.clazzName}
                    currentValue={config.current}
                    key={index}
                    label={config.label}
                    listOfValues={config.values}
                    name={config.name}
                    onSelectChange={handleModelChange}
                />],
            [R.whereEq({type: 'input'}),
                R.always(
                    <div
                        className={
                            classNames(R.replace('.', '-', R.or(sectionConfig.name, '')), sectionConfig.clazzName)
                        }
                        key={index}>
                        <span className="label">{sectionConfig.label}</span>
                        <input
                            onChange={onInputChange}
                            type="text"
                            value={sectionConfig.value}
                        />
                        {createApiButton()}
                    </div>
                )]
        ])(sectionConfig);
    };

    return bindingItems ? RU.mapIndexed(createBindingSection, bindingItems) : null;
};

export const generateTree = (arity, depth, parentIndex = '', labelPrefix = 'Node') => {
    const generateId = (index) =>
        `${R.toLower(labelPrefix)}-${parentIndex}-${index + 1}--${depth}`;
    const generateLabel = (index) =>
        `${labelPrefix} ${parentIndex}${parentIndex ? '.' : ''}${index + 1}...${depth}`;
    const generateChildren = (index) =>
        generateTree(arity, depth - 1, `${parentIndex}${parentIndex ? '.' : ''}${index + 1}`, labelPrefix);

    const createLevel = (index) => ({
        id: generateId(index),
        name: generateLabel(index)
    });

    const generateOneLevelTree = () => R.addIndex(R.map)((item, index) => createLevel(index), R.repeat(null, arity));

    const generateMultiLevelTree = () => R.addIndex(R.map)((item, index) => ({
        children: generateChildren(index),
        id: generateId(index),
        name: generateLabel(index)
    }))(R.repeat(null, arity));

    return depth === 1 ? generateOneLevelTree() : generateMultiLevelTree();
};

export const generateLazyChildren = (node) => [{
    id: `${node.id}.1`,
    loadOnDemand: true,
    name: `Lazy-Child${node.state.path}`
}];
