import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';

import {resetPristinity} from '../../utils/components-utils';
import RU from '../../utils/ramda-utils';
import {isReadOnly} from '../../utils/display-mode';

import {transformItem, transformModel} from '../../utils/transform-utils';
import {
    computeInitialState,
    computeNextState,
    getValidators
} from '../../services/component-service';
import {XlReactDipContainerProps} from '../xl-react-dip-props';
import {
    addItem,
    moveItem,
    prepareViewModel,
    revalidateItems,
    filterItems,
    sortItems,
    invalidCiValidator
} from './xl-react-dip-collection-of-ci-service';
import {XlReactDipCollectionOfCiComponent} from './xl-react-dip-collection-of-ci-component';

export class XlReactDipCollectionOfCi extends Component {
    static propTypes = {
        emptyPlaceholder: PropTypes.string,
        filterVisible: PropTypes.bool,
        handlers: PropTypes.object,
        inputVisible: PropTypes.bool,
        model: PropTypes.array,
        newItemSuffix: PropTypes.string,
        objLabel: PropTypes.string,
        openOnClick: PropTypes.bool,
        sortable: PropTypes.bool,
        sortAutomatically: PropTypes.bool,
        unique: PropTypes.bool,
        ...XlReactDipContainerProps
    };

    static defaultProps = {
        filterVisible: true,
        inputVisible: true,
        metadata: {},
        objLabel: 'id',
        validators: []
    };

    constructor(props) {
        super(props);
        const items = prepareViewModel(props.model, props.objLabel);
        this.state = {
            searchTerm: '',
            validatedItems: items,
            visibleItems: items
        };
    }

    componentWillMount() {
        computeInitialState(
            this,
            this.props,
            getValidators(this.props, [invalidCiValidator(this.state)]),
            this.getNewModel(this.props.model)
        );
    }

    componentDidMount() {
        this.reValidateAndComputeNextState(this.getNewModel(this.props.model), this.props, {pristine: true});
    }

    componentWillReceiveProps(nextProps) {
        const {pristine} = this.state;
        const model = !R.isNil(nextProps.model) ? nextProps.model : this.state.model;
        if (!R.isNil(model)) {
            const transformedModel = this.getNewModel(transformModel(this.props.objLabel)(model));
            this.reValidateAndComputeNextState(transformedModel, nextProps, {pristine});
        }
    }

    onAddItem = (item) => {
        const model = addItem(this.state.model, transformItem(item, this.props.objLabel), this.props.unique);
        this.applyNewModel(model, [() => this.autoCompleteApi.erase()]);
    };

    onFilterUpdate = (searchTerm) => {
        const {validatedItems} = this.state;
        const {objLabel} = this.props;
        this.setState({searchTerm, visibleItems: filterItems(validatedItems, searchTerm, objLabel)});
    };

    onModelChange = (items, sortable, event) => {
        this.applyNewModel(moveItem(event.oldIndex, event.newIndex, this.state.model));
    };

    onRemoveItem = (itemIndex) => {
        this.applyNewModel(R.remove(itemIndex, 1, this.state.model));
    };

    reValidateAndComputeNextState = async (newModel, props, options) => {
        await revalidateItems(this, newModel);
        computeNextState(
            this,
            props,
            getValidators(props, [invalidCiValidator(this.state)]),
            newModel,
            options
        );
    };

    api = {
        focus: () => this.autoCompleteApi.focus(),
        resetPristinity: (config) => resetPristinity(this, this.props, config, () => {
            RU.invoke(['autoCompleteApi', 'resetPristinity'], [], this);
        })
    };

    getNewModel = (model) => {
        const {objLabel, sortAutomatically} = this.props;
        return sortAutomatically ? sortItems(model, objLabel) : model;
    };

    applyNewModel = (model, customCallbacks = []) => {
        const newModel = this.getNewModel(model);
        this.reValidateAndComputeNextState(newModel, this.props, {customCallbacks});
    };

    registerAutoCompleteApi = (autoCompleteApi) => {
        this.autoCompleteApi = autoCompleteApi;
    };

    render() {
        const {
            disabled, displayMode, filterVisible, handlers, inputVisible, metadata,
            objLabel, onMessageChange, onValidityChange, sortable
        } = this.props;

        const {
            visibleItems,
            message,
            searchTerm,
            valid
        } = this.state;

        const readOnly = isReadOnly(displayMode, disabled);

        return <XlReactDipCollectionOfCiComponent {...({
            filterVisible,
            handleAddItem: this.onAddItem,
            handleFilterUpdate: this.onFilterUpdate,
            handleModelChange: this.onModelChange,
            handleRemoveItem: this.onRemoveItem,
            handlers,
            inputVisible,
            message,
            metadata,
            objLabel,
            onMessageChange,
            onValidityChange,
            readOnly,
            registerAutoCompleteApi: this.registerAutoCompleteApi,
            searchTerm,
            sortable,
            valid,
            visibleItems
        })}
        />;
    }
}
