import React from 'react';
import {mount} from 'enzyme';

import Helper from '../../validations-spec-helper';
import {XlReactDipCi} from '~/app/xl-react-dip/xl-react-dip-ci/xl-react-dip-ci';
import {ERROR} from '~/app/utils/ci-utils';

describe('XlReactDipCi-default', () => {
    const defaultProps = {
        displayMode: 'edit',
        metadata: {label: 'Test Label', required: false},
        objLabel: 'id'
    };
    it('should have all elements to be present', () => {
        const wrapper = mount(<XlReactDipCi {...defaultProps}/>);
        expect(wrapper.find('XlReactWidgetLabel')).toExist();
        expect(wrapper.find('XlReactWidgetAutocomplete')).toExist();
    });

    it('should be able to render the component without label', () => {
        const wrapper = mount(<XlReactDipCi/>);
        expect(wrapper.find('XlReactWidgetLabel')).not.toExist();
        expect(wrapper.find('.xl-react-components-input-full')).toExist();
    });

    it('should be able to render the component with label', () => {
        const wrapper = mount(<XlReactDipCi {...defaultProps}/>);
        expect(wrapper.find('XlReactWidgetLabel')).toExist();
        expect(wrapper.find('.xl-react-components-input')).toExist();
    });

    it('should be valid when term is not fully matching', () => {
        const onValidityChange = jest.fn();
        const wrapper = mount(<XlReactDipCi {...defaultProps} onValidityChange={onValidityChange}/>);
        wrapper.setState({model: 'Application'});
        expect(onValidityChange).toBeCalledWith(true);
    });

    it('should be valid when emptied', () => {
        const onValidityChange = jest.fn();
        const wrapper = mount(<XlReactDipCi {...defaultProps} onValidityChange={onValidityChange}/>);
        wrapper.setState({model: ''});
        expect(onValidityChange).toBeCalledWith(true);
    });

    it('should honour custom validators', () => {
        const validator = (model) => ({
            condition: model && !model.startsWith('/Applications'),
            message: {show: false, text: `${model} does not start with /Applications`, type: ERROR}
        });
        const {changeInput, errMsg} = new Helper('.xl-react-dip-ci')
            .buildFrom(<XlReactDipCi validators={[validator]}/>);
        changeInput('/Application/PetClinic-ear');
        expect(errMsg()).toHaveText('/Application/PetClinic-ear does not start with /Applications');
    });
});

describe('XlReactDipCi-required', () => {
    const defaultProps = {
        displayMode: 'edit',
        metadata: {required: true},
        model: '',
        objLabel: 'id'
    };
    it('should be invalid when loaded', () => {
        const onValidityChange = jest.fn();
        mount(<XlReactDipCi {...defaultProps} onValidityChange={onValidityChange}/>);
        expect(onValidityChange).toBeCalledWith(false);
    });
    it('should become valid when not empty', () => {
        const onValidityChange = jest.fn();
        const wrapper = mount(<XlReactDipCi {...defaultProps} onValidityChange={onValidityChange}/>);
        wrapper.setState({model: 'Application'});
        expect(onValidityChange).toBeCalledWith(true);
    });
});

describe('XlReactDipCi-with-given-value', () => {
    const defaultProps = {
        displayMode: 'edit',
        metadata: {},
        model: 'Application1',
        objLabel: 'id'
    };

    it('should be valid when not empty', () => {
        const onValidityChange = jest.fn();
        mount(<XlReactDipCi {...defaultProps} onValidityChange={onValidityChange}/>);
        expect(onValidityChange).toBeCalledWith(true);
    });
});
