import React from 'react';
import {mount} from 'enzyme';

import Helper from '../../validations-spec-helper';
import {XlReactDipEnum} from '~/app/xl-react-dip/xl-react-dip-enum/xl-react-dip-enum';
import {ERROR} from '~/app/utils/ci-utils';

describe('XlReactDipEnum-default', () => {
    const defaultProps = {
        metadata: {
            default: 'test-default-val',
            enumValues: ['test1', 'test2', 'test3', 'test-default'],
            label: '',
            required: false
        }
    };

    it('should have all elements present', () => {
        const defaultProps2 = {
            metadata: {
                default: 'test-default-val',
                description: 'test description',
                enumValues: ['test1', 'test2', 'test3', 'test-default'],
                label: 'enum',
                required: false
            }
        };
        const wrapper = mount(<XlReactDipEnum {...defaultProps2}/>);
        expect(wrapper.find('.xl-react-components-label')).toExist();
        expect(wrapper.find('XlReactWidgetAutocomplete')).toExist();
        expect(wrapper.find('XlReactWidgetAutocomplete .xl-react-autocomplete-container')).toExist();
        expect(wrapper.find('.xl-react-components-description')).toExist();
        expect(wrapper.find('XlReactWidgetAutocomplete input')).toExist();
    });

    it('should erase model on clear ("x") button click', () => {
        const onModelChange = jest.fn();
        const wrapper = mount(<XlReactDipEnum {...defaultProps} onModelChange={onModelChange}/>);
        wrapper.setState = {model: 'test'};
        expect(wrapper.find('XlReactWidgetAutocomplete .xl-react-btn-close')).toExist();
        wrapper.find('XlReactWidgetAutocomplete .xl-react-btn-close').simulate('click');
        expect(wrapper.find('XlReactWidgetAutocomplete .xl-react-btn-close')).not.toExist();
        expect(onModelChange).toHaveBeenCalledWith('', false);
        expect(onModelChange).toHaveBeenCalled();
    });

    it('should take default value when model is not defined', () => {
        const onModelChange = jest.fn();
        mount(<XlReactDipEnum {...defaultProps} onModelChange={onModelChange}/>);
        expect(onModelChange).toBeCalledWith('test-default-val', true);
    });

    it('should take value from model', () => {
        const wrapper = mount(<XlReactDipEnum {...defaultProps} model="not-default"/>);
        expect(wrapper.prop('model')).toBe('not-default');
    });

    it('should be valid when model is in suggestions', () => {
        const onValidityChange = jest.fn();
        mount(<XlReactDipEnum {...defaultProps} model="test1" onValidityChange={onValidityChange}/>);
        expect(onValidityChange).toBeCalledWith(true);
    });

    it('should be invalid when model not in suggestions', () => {
        const defaultProps2 = {
            metadata: {
                enumValues: ['test1', 'test2', 'test3', 'test-default'],
                label: '',
                required: false
            }
        };

        const onValidityChange = jest.fn();
        mount(<XlReactDipEnum {...defaultProps2} model="abcd" onValidityChange={onValidityChange}/>);
        expect(onValidityChange).toBeCalled();
    });

    it('should be valid when empty', () => {
        const onValidityChange = jest.fn();
        mount(<XlReactDipEnum
            metadata={{enumValues: ['test1', 'test2', 'test3', 'test-default']}}
            onValidityChange={onValidityChange}
        />);
        expect(onValidityChange).toBeCalledWith(true);
    });

    it('should honour custom validators', () => {
        const validator = (model) => ({
            condition: model === 'xxx',
            message: {show: true, text: 'You must be at least 18 years old for xxx content', type: ERROR}
        });
        const {changeInput, errMsg} = new Helper('.xl-react-dip-enum').buildFrom(<XlReactDipEnum
            metadata={{enumValues: ['xxx', 'yyy', 'zzz']}}
            model={'zzz'}
            validators={[validator]}
        />);
        changeInput('xxx');
        expect(errMsg()).toHaveText('You must be at least 18 years old for xxx content');
    });
});

describe('XlReactDipEnum-required-true', () => {
    const defaultProps = {
        displayMode: 'edit',
        metadata: {
            default: 'test-default',
            enumValues: ['test1', 'test2', 'test3', 'test-default'],
            label: 'test label',
            required: true
        }
    };

    it('should be invalid when empty', () => {
        const defaultProps2 = {
            displayMode: 'edit',
            metadata: {
                enumValues: ['test1', 'test2', 'test3', 'test-default'],
                label: 'test label',
                required: true
            }
        };
        const onValidityChange = jest.fn();
        const onModelChange = jest.fn();
        mount(<XlReactDipEnum
            {...defaultProps2}
            model=""
            onModelChange={onModelChange}
            onValidityChange={onValidityChange}
        />);
        expect(onModelChange).toBeCalledWith('', true);
        expect(onValidityChange).toBeCalledWith(false);
    });

    it('should be valid when not empty', () => {
        const onValidityChange = jest.fn();
        mount(<XlReactDipEnum {...defaultProps} model="test1" onValidityChange={onValidityChange}/>);
        expect(onValidityChange).toBeCalledWith(true);
    });
});
