import React from 'react';
import {mount} from 'enzyme';
import {waitFor} from 'async-enzyme-helpers';
import R from 'ramda';
import {Promise} from 'es6-promise';
import {ERROR} from '~/app/utils/ci-utils';
import {XlReactDipSetOfCi} from '~/app/xl-react-dip/xl-react-dip-set-of-ci/xl-react-dip-set-of-ci';

const autocompleteElements = [
    {id: 'Application1', type: '1'},
    {id: 'Application2', type: '2'},
    {id: 'Application3', type: '3'},
    {id: 'Application4', type: '4'},
    {id: 'Application5', type: '5'},
    {id: 'Application6', type: '6'}];

const addCandidates = (metadata, value) => {
    const filterCond = (item) => !value || R.toLower(item.id).indexOf(R.toLower(value)) !== -1;
    const res = R.filter(filterCond, autocompleteElements);
    return Promise.resolve(res);
};

const selector = 'SortableList';

const getSetOfCiHandlers = () => {
    const setOfCiEvents = [];

    return {
        addCandidates,
        onClick: (value) => {
            setOfCiEvents.push({message: `OnClick: ${JSON.stringify(value)}`});
        },
        onNewItem: (value) => {
            setOfCiEvents.push({message: `OnNewItem: ${JSON.stringify(value)}`});
        }
    };
};

const model = ['Item 1', 'Item 2', 'Item 3', 'Item 4'];

describe('xl-dip-set-of-ci', () => {
    describe('XlReactDipSetOfCi-default', () => {
        const defaultProps = {
            handlers: getSetOfCiHandlers(),
            metadata: {
                description: 'test desc',
                label: 'test label',
                name: 'test name',
                required: false
            },
            model: []
        };

        it('should have all elements present', () => {
            const onModelChange = jest.fn();
            const onValidityChange = jest.fn();
            const wrapper = mount(<XlReactDipSetOfCi
                {...defaultProps}
                model={model}
                onModelChange={onModelChange}
                onValidityChange={onValidityChange}
            />);
            expect(wrapper.find('XlReactDipCollectionOfCi')).toExist();
            expect(wrapper.find('XlReactDipCollectionOfCi .xl-react-autocomplete-container')).toExist();
            expect(wrapper.find('XlReactWidgetLabel')).toExist();
            expect(wrapper.find('XlReactWidgetLabel label')).toHaveText(defaultProps.metadata.label);
            expect(wrapper.find('.xl-react-components-description')).toExist();
            expect(wrapper.find('.xl-react-components-description')).toHaveText(defaultProps.metadata.description);
            expect(wrapper.find('XlReactWidgetAutocomplete')).toExist();
            expect(wrapper.find('XlReactWidgetFilter')).toExist();
            expect(onModelChange).toBeCalledWith(['Item 1', 'Item 2', 'Item 3', 'Item 4'], true);
            expect(wrapper.find('SortableList').prop('canSort')).toEqual(false);
            expect(wrapper.find('SortableList ul')).not.toHaveClassName('ui-sortable');
            const items = wrapper.find('SortableList .xl-list-row');
            expect(items.length).toEqual(4);

            const input = wrapper.find('XlReactDipCollectionOfCi .xl-react-autocomplete-container input[type="text"]');
            expect(input).toExist();
            input.simulate('change', {target: {value: 'A'}});
            expect(wrapper.find('XlReactWidgetAutocomplete .xl-react-btn-close')).toExist();
        });

        it('should have delete buttons for all list items', () => {
            const wrapper = mount(<XlReactDipSetOfCi
                {...defaultProps}
                model={['Item 1', 'Item 2', 'Item 3', 'Item 4']}
            />);
            const items = wrapper.find('SortableList .xl-list-row .xl-react-btn-close');
            expect(items.length).toEqual(4);
        });

        it('should remove elements one by one', async () => {
            const onModelChange = jest.fn();
            const wrapper = mount(<XlReactDipSetOfCi
                {...defaultProps}
                model={model}
                onModelChange={onModelChange}
            />);
            expect(onModelChange).toBeCalledWith(['Item 1', 'Item 2', 'Item 3', 'Item 4'], true);
            const deleteButtons = () => wrapper.find(`${selector} .xl-list-row .xl-react-btn-close`);

            deleteButtons().first().simulate('click');
            await waitFor(wrapper, selector);
            expect(onModelChange).toBeCalledWith(['Item 2', 'Item 3', 'Item 4'], false);

            deleteButtons().first().simulate('click');
            await waitFor(wrapper, selector);
            expect(onModelChange).toBeCalledWith(['Item 3', 'Item 4'], false);

            deleteButtons().first().simulate('click');
            await waitFor(wrapper, selector);
            expect(onModelChange).toBeCalledWith(['Item 4'], false);

            deleteButtons().first().simulate('click');
            await waitFor(wrapper, selector);
            expect(onModelChange).toBeCalledWith([], false);
        });

        it('should honour custom validators', async () => {
            const validator = (componentModel) => ({
                condition: componentModel.length < 4,
                message: {show: false, text: 'At least 4 items required', type: ERROR}
            });
            const wrapper = mount(<XlReactDipSetOfCi
                {...defaultProps}
                model={model}
                validators={[validator]}
            />);
            const deleteButtons = () => wrapper.find(`${selector} .xl-list-row .xl-react-btn-close`);

            deleteButtons().first().simulate('click');
            await waitFor(wrapper, '.error');

            expect(wrapper.find('.error')).toHaveText('At least 4 items required');
        });

        it('should clear input field with close button', () => {
            const wrapper = mount(<XlReactDipSetOfCi {...defaultProps}/>);
            const input = wrapper.find('XlReactDipCollectionOfCi .xl-react-autocomplete-container input[type="text"]');
            expect(input).toExist();
            input.simulate('change', {target: {value: 'AA'}});
            expect(wrapper.find('XlReactWidgetAutocomplete .xl-react-btn-close')).toExist();
            input.simulate('change', {target: {value: ''}});
            expect(wrapper.find('XlReactWidgetAutocomplete .xl-react-btn-close')).not.toExist();
        });

        it("Filter should be/should'nt be Visible based on 'filterVisible' prop  ", () => {
            const wrapper = mount(<XlReactDipSetOfCi {...defaultProps} displayMode="edit"/>);
            expect(wrapper.find('XlReactWidgetFilter input[type="text"]')).toExist();

            wrapper.setProps({filterVisible: false});
            expect(wrapper.find('XlReactWidgetFilter input[type="text"]')).not.toExist();
            wrapper.setProps({filterVisible: true});
            expect(wrapper.find('XlReactWidgetFilter input[type="text"]')).toExist();
        });

        it('should filter items', () => {
            const wrapper = mount(<XlReactDipSetOfCi
                {...defaultProps}
                model={model}
            />);
            const filterInput = wrapper.find('XlReactWidgetFilter input[type="text"]');
            const items = () => wrapper.find('SortableList .xl-list-row');
            expect(items().length).toEqual(4);

            filterInput.simulate('change', {target: {value: '1'}});
            expect(items().length).toEqual(1);

            filterInput.simulate('change', {target: {value: 'item'}});
            expect(items().length).toEqual(4);

            filterInput.simulate('change', {target: {value: 'zzz'}});
            expect(items().length).toEqual(0);
        });

        it('should display all item when filter is emptied', () => {
            const wrapper = mount(<XlReactDipSetOfCi
                {...defaultProps}
                model={model}
            />);
            const filterInput = wrapper.find('XlReactWidgetFilter input[type="text"]');
            const items = () => wrapper.find('SortableList .xl-list-row');
            expect(items().length).toEqual(4);

            filterInput.simulate('change', {target: {value: 'Item 1'}});
            expect(items().length).toEqual(1);

            filterInput.simulate('change', {target: {value: ''}});
            expect(items().length).toEqual(4);
        });

        it('should/nt be disabled', () => {
            const wrapper = mount(<XlReactDipSetOfCi
                {...defaultProps}
                disabled={false}
            />);
            expect(wrapper.find('.xl-react-autocomplete-container input[disabled="true"]')).not.toExist();
            wrapper.setProps({disabled: true});
            expect(wrapper.find('.xl-react-autocomplete-container input[disabled]')).toExist();
        });
    });

    describe('XlReactDipSetOfCi-required', () => {
        const defaultProps = {
            handlers: getSetOfCiHandlers(),
            metadata: {
                description: 'test desc',
                label: 'test label',
                name: 'test name',
                required: true
            }
        };

        it('should be invalid when model empty', async () => {
            const onModelChange = jest.fn();
            const onValidityChange = jest.fn();
            const wrapper = mount(<XlReactDipSetOfCi
                {...defaultProps}
                model={[]}
                onModelChange={onModelChange}
                onValidityChange={onValidityChange}
            />);
            expect(onValidityChange).toBeCalledWith(false);
            await waitFor(wrapper, selector);

            wrapper.setProps({model: [{id: 'chintu', type: '10'}]});
            await waitFor(wrapper, selector);

            expect(onValidityChange).toBeCalledWith(true);
        });

        it('should become invalid from a valid state when emptied', async () => {
            const onModelChange = jest.fn();
            const onValidityChange = jest.fn();
            const wrapper = mount(<XlReactDipSetOfCi
                {...defaultProps}
                model={[{id: 'chintu', type: '10'}]}
                onModelChange={onModelChange}
                onValidityChange={onValidityChange}
            />);
            await waitFor(wrapper, selector);
            expect(onValidityChange).toBeCalledWith(true);
            wrapper.setProps({model: []});
            await waitFor(wrapper, selector);

            expect(onValidityChange).toBeCalledWith(false);
        });
    });

    describe('XlReactDipSetOfCi-displayMode', () => {
        const defaultProps = () => ({
            handlers: getSetOfCiHandlers(),
            metadata: {
                default: ['item1', 'item3', 'item3'],
                description: 'test desc',
                label: 'test label',
                name: 'test name',
                required: false
            }
        });

        it('should switch display mode', async () => {
            const onModelChange = jest.fn();
            const wrapper = mount(<XlReactDipSetOfCi
                {...defaultProps()}
                model={model}
                onModelChange={onModelChange}
            />);
            const input = wrapper.find('.xl-react-autocomplete-container input[type="text"]');
            const deleteButtons = () => wrapper.find('SortableList .xl-list-row .xl-react-btn-close');
            wrapper.setProps({displayMode: ''});
            await waitFor(wrapper, 'input');

            expect(input).toExist();
            expect(deleteButtons().length).toBe(4);
            expect(wrapper.find('SortableList ul')).not.toHaveClassName('ui-sortable');

            wrapper.setProps({displayMode: 'edit'});
            expect(input).toExist();
            expect(deleteButtons().length).toBe(4);

            wrapper.setProps({displayMode: 'visual'});
            expect(wrapper.find('.xl-react-autocomplete-container input[type="text"][disabled]')).toExist();
            expect(wrapper.find('.xl-btn-add')).not.toExist();

            wrapper.setProps({
                disabled: false,
                displayMode: 'edit'
            });
            wrapper.setProps({disabled: true});
            expect(wrapper.find('.xl-react-autocomplete-container input[type="text"][disabled]')).toExist();
            expect(deleteButtons()).not.toExist();
            expect(wrapper.find('SortableList ul')).not.toHaveClassName('ui-sortable');
        });

        it('should not be able to edit in Visual mode', () => {
            const onModelChange = jest.fn();
            const wrapper = mount(<XlReactDipSetOfCi
                {...defaultProps()}
                displayMode="visual"
                model={model}
                onModelChange={onModelChange}
            />);
            const input = wrapper.find('.xl-react-autocomplete-container input[type="text"][disabled]');
            const deleteButtons = () => wrapper.find('SortableList .xl-list-row .xl-react-btn-close');
            const rows = wrapper.find('SortableList .xl-list-row');

            expect(input).toExist();
            expect(rows.length).toEqual(4);
            expect(deleteButtons()).not.toExist();
            expect(wrapper.find('SortableList ul')).not.toHaveClassName('ui-sortable');
        });

        it('should be able to filter list in Visual mode', async () => {
            const onModelChange = jest.fn();
            const wrapper = mount(<XlReactDipSetOfCi
                {...defaultProps()}
                displayMode="visual"
                model={model}
                onModelChange={onModelChange}
            />);

            const input = () => wrapper.find('.xl-react-autocomplete-container input[type="text"][disabled]');
            const deleteButtons = () => wrapper.find('SortableList .xl-list-row .xl-react-btn-close');
            const items = () => wrapper.find('SortableList .xl-list-row');
            const filterInput = () => wrapper.find('XlReactWidgetFilter input[type="text"]');

            await waitFor(wrapper, 'SortableList .xl-list-row');
            expect(input()).toExist();
            expect(items().length).toEqual(4);
            expect(deleteButtons()).not.toExist();
            expect(wrapper.find('SortableList').prop('canSort')).toEqual(false);
            expect(wrapper.find('SortableList ul')).not.toHaveClassName('ui-sortable');

            filterInput().simulate('change', {target: {value: '1'}});
            expect(items().length).toEqual(1);

            filterInput().simulate('change', {target: {value: 'item'}});
            expect(items().length).toEqual(4);

            filterInput().simulate('change', {target: {value: 'zzz'}});
            expect(items().length).toEqual(0);

            filterInput().simulate('change', {target: {value: ''}});
            expect(items().length).toEqual(4);
        });
    });
});
