import R from 'ramda';
import metadata from './application-metadata.json';
import {groupedProperties} from './grouped-properties.json';
import {groupedProperties as updatedGroupedProperties} from './updated-grouped-properties.json';
import {groupedProperties as openedGroupedProperties} from './expanded-grouped-properties.json';
import {groupedProperties as collapsedGroupedProperties} from './collapsed-grouped-properties.json';
import formActions
    from '~/app/xl-react-form/xl-react-dip-redux-form/xl-react-dip-redux-form-actions';
import {xlReactDipReduxFormReducer}
    from '~/app/xl-react-form/xl-react-dip-redux-form/xl-react-dip-redux-form-reducer';

describe('xl-react-dip-redux-form-reducer', () => {
    describe('xlReactDipReduxFormReducer', () => {
        it('should initialize redux store with initial form data', () => {
            expect(xlReactDipReduxFormReducer({})({}, formActions.initialize({
                collapsed: 'auto',
                formId: 'form-1',
                metadata,
                pristine: true
            }))).toEqual({
                'form-1': {
                    collapsed: 'auto',
                    formId: 'form-1',
                    groupedProperties: collapsedGroupedProperties,
                    metadata,
                    model: {
                        lastVersion: '',
                        pipeline: '',
                        verifyChecklistPermissionsOnCreate: false
                    },
                    pristine: true
                }
            });
        });
    });
    describe('mergeContext', () => {
        it('should merge new context with already existing one', () => {
            const updatedMetadata = {
                ...metadata,
                properties: R.map((property) => ({...property, kind: 'BOOLEAN'}), metadata.properties)
            };

            expect(xlReactDipReduxFormReducer({})({
                'form-1': {
                    collapsed: true,
                    formId: 'form-1',
                    metadata
                }
            }, formActions.mergeContext('form-1',
                {metadata: updatedMetadata}
            ))).toEqual({
                'form-1': {
                    collapsed: true,
                    formId: 'form-1',
                    groupedProperties: updatedGroupedProperties,
                    metadata: updatedMetadata
                }
            });
        });
    });
    describe('openCategoryGroup', () => {
        it('should open all category group in grouped properties', () => {
            expect(xlReactDipReduxFormReducer({})({
                'form-1': {
                    collapsed: true,
                    formId: 'form-1',
                    groupedProperties,
                    metadata
                }
            }, formActions.openCategoryGroup('form-1', 'Common'))).toEqual({
                'form-1': {
                    collapsed: true,
                    formId: 'form-1',
                    groupedProperties: openedGroupedProperties,
                    metadata
                }
            });
        });
    });
    describe('toggleCategoryGroup', () => {
        it('should toggle category group in grouped properties', () => {
            expect(xlReactDipReduxFormReducer({})({
                'form-1': {
                    collapsed: true,
                    formId: 'form-1',
                    groupedProperties: openedGroupedProperties,
                    metadata
                }
            }, formActions.toggleCategoryGroup('form-1', 'Common'))).toEqual({
                'form-1': {
                    collapsed: true,
                    formId: 'form-1',
                    groupedProperties,
                    metadata
                }
            });
        });
    });
    describe('unmount', () => {
        it('should unmount all data for specified form', () => {
            expect(xlReactDipReduxFormReducer({})({
                'form-1': {
                    collapsed: true,
                    formId: 'form-1',
                    groupedProperties: openedGroupedProperties,
                    metadata
                },
                'form-2': {
                    collapsed: true,
                    formId: 'form-1',
                    groupedProperties: openedGroupedProperties,
                    metadata
                }
            }, formActions.unmount('form-1'))).toEqual({
                'form-2': {
                    collapsed: true,
                    formId: 'form-1',
                    groupedProperties: openedGroupedProperties,
                    metadata
                }
            });
        });
    });
    describe('changePristineState', () => {
        it('should set pristine state for specified form', () => {
            const form1 = {
                collapsed: true,
                formId: 'form-1',
                groupedProperties: openedGroupedProperties,
                metadata
            };

            const pristinity1 = {condition: false, path: true};
            const pristinity2 = {condition: true, path: true};

            expect(xlReactDipReduxFormReducer({})(
                {'form-1': form1},
                formActions.changePristineState('form-1', pristinity1))
            ).toEqual({'form-1': {...form1, pristine: false, pristinity: pristinity1}});

            expect(xlReactDipReduxFormReducer({})(
                {'form-1': form1},
                formActions.changePristineState('form-1', pristinity2))
            ).toEqual({'form-1': {...form1, pristine: true, pristinity: pristinity2}});
        });
    });
    describe('changeValidity', () => {
        it('should set validity for specified form', () => {
            const form1 = {
                collapsed: true,
                formId: 'form-1',
                groupedProperties: openedGroupedProperties,
                metadata
            };

            const validity1 = {condition: false, path: true};
            const validity2 = {condition: true, path: true};

            expect(xlReactDipReduxFormReducer({})(
                {'form-1': form1},
                formActions.changeValidity('form-1', validity1))
            ).toEqual({'form-1': {...form1, valid: false, validity: validity1}});

            expect(xlReactDipReduxFormReducer({})(
                {'form-1': form1},
                formActions.changeValidity('form-1', validity2))
            ).toEqual({'form-1': {...form1, valid: true, validity: validity2}});
        });
    });
});
